void bshield(void) {

#include <TMath.h>
#include <TRandom.h>

gROOT->Reset();
//TTree *Bfield = (TTree *) gROOT->FindObject("Bfield");
gStyle->SetPalette(1,0);
gStyle->SetOptStat(kFALSE);
gStyle->SetPadRightMargin(0.15);
gStyle->SetPadLeftMargin(0.15);
gStyle->SetPadBottomMargin(0.15);
//gStyle->SetTitleFontSize(0.07);
//gStyle->SetLabelSize(0.05);
//gStyle->SetTextSize(0.05);
//
   char string[256];
   double mu;

   TCanvas *c1 = new TCanvas("c1","Formulas for cylindrical mu-metal shield",200,10,700,500);
   c1->SetGridx();
   c1->SetGridy();
   double xmin=1;
   double xmax=10000;
   double ymin=0.01;
   double ymax=100000;
   //
   // We create a formula object and compute the value of this formula
   // for two different values of the x variable.
   //
   // size for 3" tubes measured in CLAS-NOTE 93-008.
   // double a=4.03;
   // double b=4.13;

   // Assume 2.25" shield, 1mm thick
   //double a=2.56;
   //double a=2.81;
   //double b=2.86;

   // Assume 2." shield, 0.356 mm thick
   double a=2.504;
   double b=2.54;

   // permeability for AD-MU-78

   double Km0=Km=43000;
   //double mu0=1.257e-6;
   //double H0=100/mu0;
   double H0=30;

   // note that Bsaturation is also defined in function Hx

   // double BSaturation=7600;  
   // divide by empirical factor of 2
   double BSaturation=3800;
 
   TF1 *D = new TF1("D",Delta,xmin,xmax,4);
   D->SetParName(0,"a");
   D->SetParName(1,"b");
   D->SetParName(2,"Km");
   D->SetParName(3,"H0");
   D->SetParameters(a,b,Km,H0);
   gPad->SetLogx(1);
   gPad->SetLogy(1);
   D->GetYaxis()->SetRangeUser(ymin,ymax);
   D->GetXaxis()->SetTitle("Magnetic Field Strength (Oe)");
   D->GetYaxis()->SetTitle("B1, B2M, A2M, A3M");
   D->SetLineColor(4);
   // D->Draw();
   sprintf(string," Inner Radius=%.1f cm, Outer Radius=%.1f cm\n",a,b);
   printf("string=%s",string);
   t1 = new TLatex(0.2,0.8,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   // t1->Draw();
   //

   TF1 *B1 = new TF1("B1",B1,xmin,xmax,4);
   B1->SetParName(0,"a");
   B1->SetParName(1,"b");
   B1->SetParName(2,"Km");
   B1->SetParName(3,"H0");
   B1->SetParameters(a,b,Km,H0);
   gPad->SetLogx(1);
   gPad->SetLogy(1);
   B1->GetYaxis()->SetRangeUser(ymin,ymax);
   B1->GetXaxis()->SetTitle("Magnetic Field Strength (Oe)");
   B1->GetYaxis()->SetTitle("B1, B2M, A2M, A3M");
   B1->SetLineColor(2);
   B1->Draw();
   sprintf(string," Inner Radius=%.1f cm, Outer Radius=%.1f cm, #mu=%.0f, H0=%.2f\n",a,b,Km,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();

   TF1 *B2M = new TF1("B2M",B2M,xmin,xmax,4);
   B2M->SetParameters(a,b,Km,H0); 
   B2M->SetLineColor(1);
   B2M->Draw("same");

   TF1 *A2M = new TF1("A2M",A2M,xmin,xmax,4);
   A2M->SetParameters(a,b,Km,H0); 
   A2M->SetLineColor(3);
   A2M->Draw("same");

   TF1 *A3M = new TF1("A3M",A3M,xmin,xmax,4);
   A3M->SetParameters(a,b,Km,H0); 
   A3M->SetLineColor(4);
   A3M->Draw("same");

   TF1 *check = new TF1("check",check,xmin,xmax,4);
   check->SetParameters(a,b,Km,H0); 
   check->SetLineColor(1);
   check->Draw("same");

   c1->SaveAs("bshield.eps");
   c1->SaveAs("bshield.gif");

   TCanvas *c2 = new TCanvas("c2","Magnetic Field Strength for cylindrical mu-metal shield",200,10,700,500);
   c2->SetGridx();
   c2->SetGridy();
   double xmin=-40;
   double xmax=40;
   double ymin=-40;
   double ymax=40;
   double zmin=-1000;
   double zmax=1000;

   // plot the field strength H(x,y) as a function of x and y
   // a and b are the inner and outer radii of the cylindrical shield, respectively.
   // Km is the relative permeability

   TF2 *Vm = new TF2("Vm",Vm,xmin,xmax,ymin,ymax,4);
   Vm->SetParName(0,"a");
   Vm->SetParName(1,"b");
   Vm->SetParName(2,"Km");
   Vm->SetParName(3,"H0");
   Vm->SetParameters(a,b,Km,H0);
   //gPad->SetLogz(1);
   //Vm->GetZaxis()->SetRangeUser(zmin,zmax);
   Vm->SetRange(xmin,ymin,zmin,xmax,ymax,zmax);
   Vm->GetXaxis()->SetTitle("x (cm)");
   Vm->GetYaxis()->SetTitle("y (cm)");
   Vm->SetLineColor(2);
   Vm->Draw("colz");
   sprintf(string," Inner Radius=%.1f cm, Outer Radius=%.1f cm, #mu=%.0f, H0=%.2f\n",a,b,Km,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();

   // 
   TCanvas *c3 = new TCanvas("c3","Magnetic Field Strength x-axis for cylindrical mu-metal shield",200,10,700,500);
   c3->SetGridx();
   c3->SetGridy();
   double xmin=-10;
   double xmax=10;
   double ymin=-10;
   double ymax=10;
   double zmin=0.1;
   double zmax=100000;

   TF2 *Hx = new TF2("Hx",Hx,xmin,xmax,ymin,ymax,4);
   Hx->SetParName(0,"a");
   Hx->SetParName(1,"b");
   Hx->SetParName(2,"Km");
   Hx->SetParName(3,"H0");
   Hx->SetParameters(a,b,Km,H0);


   //gPad->SetLogz(1);
   //Hx->GetZaxis()->SetRangeUser(zmin,zmax);
   //Hx->SetRange(xmin,ymin,zmin,xmax,ymax,zmax);
   Hx->GetXaxis()->SetTitle("x (cm)");
   Hx->GetYaxis()->SetTitle("y (cm)");
   Hx->SetLineColor(2);
   Hx->Draw("colz");
   sprintf(string," Inner Radius=%.1f cm, Outer Radius=%.1f cm, #mu=%.0f, H0=%.2f\n",a,b,Km,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();

   // 
   TCanvas *c4 = new TCanvas("c4","Magnetic Field Strength y-axis for cylindrical mu-metal shield",200,10,700,500);
   c4->SetGridx();
   c4->SetGridy();

   TF2 *Hy = new TF2("Hy",Hy,xmin,xmax,ymin,ymax,4);
   Hy->SetParName(0,"a");
   Hy->SetParName(1,"b");
   Hy->SetParName(2,"Km");
   Hy->SetParName(3,"H0");
   Hy->SetParameters(a,b,Km,H0);
   //gPad->SetLogz(1);
   //Hy->GetZaxis()->SetRangeUser(zmin,zmax);
   //Hy->SetRange(xmin,ymin,zmin,xmax,ymax,zmax);
   Hy->GetXaxis()->SetTitle("x (cm)");
   Hy->GetYaxis()->SetTitle("y (cm)");
   Hy->SetLineColor(2);
   Hy->Draw("colz");
   sprintf(string," Inner Radius=%.1f cm, Outer Radius=%.1f cm, #mu=%.0f, H0=%.2f\n",a,b,Km,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
 
   Int_t nxbins=160;
   Int_t nybins=160;
   Double_t deltax;
   Double_t deltay;
   Double_t xbin;
   Double_t ybin;

   // Check for saturation

   int j;
   int jmax=180;
   double theta;
   double radius;
   double xval;
   double yval;
   double Hmux;
   double Hmuy;
   double Hmu;
   double Hmu_min=10000;
   double Hmu_max=0;

   for (j=0;j<jmax;j++) {
      theta = j*3.14159/180;
      radius = (a+b)/2;
      xval = radius*cos(theta);
      yval = radius*sin(theta);
      Hmux = Hx->Eval(xval,yval);
      Hmuy = Hy->Eval(xval,yval);
      Hmu = sqrt(Hmux*Hmux + Hmuy*Hmuy);
      Hmu_min = Hmu<Hmu_min?Hmu:Hmu_min;
      Hmu_max = Hmu>Hmu_max?Hmu:Hmu_max;
      //printf ("j=%d, theta=%f, xval=%f, yval=%f, Hmu=%f, Hmu_min=%f, Hmu_max=%f\n",j,theta,xval,yval,Hmu,Hmu_min,Hmu_max);
   }
   Hmux = Hx->Eval(0,(a+b)/2);
   Hmuy = Hy->Eval(0,(a+b)/2);
   Hmu = sqrt(Hmux*Hmux + Hmuy*Hmuy);
   Double_t Bmax = Km*Hmu;
   printf ("Hmux=%f, Hmuy=%f, Hmu=%f, Bmax=%f\n",Hmux,Hmuy,Hmu,Bmax);
   

   // calculate mu from parameterization

   double pa = 140000;
   double pb = 1e10;
   double pc= 1e6;
   double pd= 4;
   double mu_calc = (pa+pb*Hmu*Hmu)/(pd+pc*Hmu*Hmu*Hmu);
   double mu_min = (pa+pb*Hmu_min*Hmu_min)/(pd+pc*Hmu_min*Hmu_min*Hmu_min);
   double mu_max = (pa+pb*Hmu_max*Hmu_max)/(pd+pc*Hmu_max*Hmu_max*Hmu_max);
   double Bmax_calc = mu_calc*Hmu;
   double Bmax_mumin = mu_min*Hmu_min;
   double Bmax_mumax = mu_max*Hmu_max;
   printf ("H0=%f, Km=%f, Hmu=%f, Bmax=%f, BSaturation=%f mu_calc=%f Bmax_calc=%f\n",H0,Km,Hmu,Bmax,BSaturation,mu_calc,Bmax_calc);
   printf ("Km=%f, mu_calc=%f, mu_min=%f, mu_max=%f\n",Km, mu_calc,mu_min,mu_max);

   // use minimum calculated mu

   Km = Km>mu_calc?mu_calc:Km;
   Km = Km>mu_min?mu_min:Km;
   Km = Km>mu_max?mu_max:Km;
   Km = Km>BSaturation/Hmu?BSaturation/Hmu:Km;
   double Bmin = Km*Hmu;
   printf ("Bmin=%f, BSaturation=%f, Km=%f\n",Bmin,BSaturation,Km);

   // reset parameter Km

   Hx->SetParameter("Km",Km);
   Hy->SetParameter("Km",Km);

   TH2F *histx = new TH2F("histx","Magnetic Field Strength (Oe) along x",nxbins,xmin,xmax,nybins,ymin,ymax);
   //TH2F *histy = new TH2F("histy","ABS(Magnetic Field Strength) (Oe) along y",nxbins,xmin,xmax,nybins,ymin,ymax);
   TH2F *histy = new TH2F("histy","Magnetic Field Strength  (Oe) along y",nxbins,xmin,xmax,nybins,ymin,ymax);
   TH2F *histBx = new TH2F("histBx","Magnetic Field (G) along x",nxbins,xmin,xmax,nybins,ymin,ymax);
   TH2F *histBtot = new TH2F("histBtot","Magnetic Field (G)",nxbins,xmin,xmax,nybins,ymin,ymax);
   //TH2F *histBy = new TH2F("histBy","ABS(Magnetic Field) (G) along y",nxbins,xmin,xmax,nybins,ymin,ymax);
   TH2F *histBy = new TH2F("histBy","Magnetic Field (G) along y",nxbins,xmin,xmax,nybins,ymin,ymax);


   for (Int_t i=0; i<nxbins; i++){
      for (Int_t j=0; j<nybins; j++) {
          deltax = (xmax-xmin)/nxbins;
          deltay = (ymax-ymin)/nybins;
          xbin = xmin + deltax*i;
          ybin = ymin + deltay*j;
          histx->Fill(xbin,ybin,Hx->Eval(xbin,ybin));
          histy->Fill(xbin,ybin,Hy->Eval(xbin,ybin));

          // use absolute value in order to use log scale
          //histy->Fill(xbin,ybin,fabs(Hy->Eval(xbin,ybin)));

          Double_t r = sqrt(xbin*xbin + ybin*ybin);
          if (r>a && r<b) {
              histBx->Fill(xbin,ybin,Km*Hx->Eval(xbin,ybin));
              //histBy->Fill(xbin,ybin,Km*fabs(Hy->Eval(xbin,ybin)));}
              histBy->Fill(xbin,ybin,Km*Hy->Eval(xbin,ybin));
              histBtot->Fill(xbin,ybin,Km*sqrt(Hx->Eval(xbin,ybin)*Hx->Eval(xbin,ybin)+ Hy->Eval(xbin,ybin)*Hy->Eval(xbin,ybin)));}
          else {
              histBx->Fill(xbin,ybin,Hx->Eval(xbin,ybin));
              //histBy->Fill(xbin,ybin,fabs(Hy->Eval(xbin,ybin)));}
              histBy->Fill(xbin,ybin,Hy->Eval(xbin,ybin));
              histBtot->Fill(xbin,ybin,sqrt(Hx->Eval(xbin,ybin)*Hx->Eval(xbin,ybin)+ Hy->Eval(xbin,ybin)*Hy->Eval(xbin,ybin)));}
      }
   }
   
   // set up canvas with four plots

   c9 = new TCanvas("c9","Cylindrical Magnetic Shield",100,10,750,750);
   sprintf(string," Inner Radius=%.2f cm, Outer Radius=%.2f cm, #mu=%.0f, B0=%.2f G\n",a,b,Km0,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.05,0.96,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw(); 
   c9->Divide(1,1,0.05,0.05);
   c9_1->Divide(2,2);

   TCanvas *c5 = new TCanvas("c5","Magnetic Field Strength x-axis for cylindrical mu-metal shield",200,10,700,700);
   c5->SetGridx();
   c5->SetGridy();
   double zmin2=0.1;
   double zmax2=3*H0;
   gPad->SetLogz(1);
   //double zmin=0;
   //double zmax=1.2;

   histx->GetZaxis()->SetRangeUser(zmin2,zmax2);
   histx->GetXaxis()->SetTitle("x (cm)");
   histx->GetYaxis()->SetTitle("y (cm)");
   histx->SetLineColor(2);
   histx->Draw("colz");
   sprintf(string," Inner R=%.2f cm, Outer R=%.2f cm, #mu=%.0f, H0=%.2f Oe\n",a,b,Km,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   sprintf(string,"Hx=%.2f Oe\n",Hx->Eval(0,0));
   printf("string=%s",string);
   t1 = new TLatex(0.42,0.52,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   TPad * c9_1 = (TPad *)(c9->GetPrimitive("c9_1"));
   c9_1->cd(1);
   gPad->SetLogz(1);
   histx->Draw("colz");
   
   c6 = new TCanvas("c6","Magnetic Field Strength y-axis for cylindrical mu-metal shield",200,10,700,700);
   //c6 = new TCanvas("c6","Abs(Magnetic Field Strength) y-axis for cylindrical mu-metal shield",200,10,700,700);
   c6->SetGridx();
   c6->SetGridy();
   double zmin=-1.5*H0;
   double zmax=1.5*H0;
   //gPad->SetLogz(1);
   histy->GetZaxis()->SetRangeUser(zmin,zmax);
   histy->Draw("colz");
   sprintf(string," Inner R=%.2f cm, Outer R=%.2f cm, #mu=%.0f, H0=%.2f Oe\n",a,b,Km,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   c9_1->cd(2);
   //gPad->SetLogz(1);
   histy->Draw("colz");

   
   TCanvas *c7 = new TCanvas("c7","Magnetic Field x-axis for cylindrical mu-metal shield",200,10,700,700);
   c7->SetGridx();
   c7->SetGridy();
   //double zmin2=0.1;
   //double zmax2=80;
   gPad->SetLogz(1);
   //double zmin=0;
   //double zmax=1.2;

   histBx->GetZaxis()->SetRangeUser(zmin2,zmax2);
   histBx->GetXaxis()->SetTitle("x (cm)");
   histBx->GetYaxis()->SetTitle("y (cm)");
   histBx->SetLineColor(2);
   histBx->Draw("colz");
   sprintf(string," Inner R=%.2f cm, Outer R=%.2f cm, #mu=%.0f, B0=%.2f G\n",a,b,Km0,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();

   c9_1->cd(3);
   gPad->SetLogz(1);
   histBx->Draw("colz");
   sprintf(string,"Bin=%.2f Gauss\n",Hx->Eval(0,0));
   printf("string=%s",string);
   t1 = new TLatex(0.2,0.86,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   sprintf(string,"B0/Bin=%.0f\n",H0/Hx->Eval(0,0));
   t1 = new TLatex(0.2,0.82,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   sprintf(string,"#mu_{eff}=%.0f\n",Km);
   t1 = new TLatex(0.2,0.78,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
 
   Double_t s = Km0*(1-(a*a)/(b*b))/4;
   Double_t s_eff = Km*(1-(a*a)/(b*b))/4;
   printf ("s=%f, s_eff=%f Km=%f, Km0=%f\n",s,s_eff,Km,Km0);
   sprintf(string,"Shield Eff Formla=%.0f\n",s);
   t1 = new TLatex(0.2,0.74,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   sprintf(string,"Bmax/Bsat=%.0f/%.0f\n",Bmax,BSaturation);
   t1 = new TLatex(0.2,0.70,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   
   //c8 = new TCanvas("c8","fabs(Magnetic Field) y-axis for cylindrical mu-metal shield",200,10,700,700);
   c8 = new TCanvas("c8","Magnetic Field y-axis for cylindrical mu-metal shield",200,10,700,700);
   c8->SetGridx();
   c8->SetGridy();
   //double zmin=-60;
   //double zmax=60;
   //double zmin=0.1;
   //double zmax=1000;
   //gPad->SetLogz(1);
   histBy->GetZaxis()->SetRangeUser(zmin,zmax);
   histBy->Draw("colz");
   sprintf(string," Inner R=%.2f cm, Outer R=%.2f cm, #mu=%.0f, B0=%.2f G\n",a,b,Km,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   c9_1->cd(4);
   //gPad->SetLogz(1);
   histBy->Draw("colz");

   TCanvas *c10 = new TCanvas("c10","Magnetic Field Total for cylindrical mu-metal shield",200,10,700,700);
   c10->SetGridx();
   c10->SetGridy();
   //double zmin2=0.1;
   //double zmax2=80;
   gPad->SetLogz(1);
   //double zmin=0;
   //double zmax=1.2;

   histBtot->GetZaxis()->SetRangeUser(zmin2,zmax2);
   histBtot->GetXaxis()->SetTitle("x (cm)");
   histBtot->GetYaxis()->SetTitle("y (cm)");
   histBtot->SetLineColor(2);
   histBtot->Draw("colz");
   sprintf(string," Inner R=%.2f cm, Outer R=%.2f cm, #mu=%.0f, B0=%.2f G\n",a,b,Km0,H0);
   printf("string=%s",string);
   t1 = new TLatex(0.15,0.91,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();

   sprintf(string,"Bin=%.2f Gauss\n",Hx->Eval(0,0));
   printf("string=%s",string);
   t1 = new TLatex(0.2,0.86,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   sprintf(string,"B0/Bin=%.0f\n",H0/Hx->Eval(0,0));
   t1 = new TLatex(0.2,0.82,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   sprintf(string,"#mu_{eff}=%.0f\n",Km);
   t1 = new TLatex(0.2,0.78,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();  
 
   Double_t s = Km0*(1-(a*a)/(b*b))/4;
   sprintf(string,"Shield Eff Formla=%.0f\n",s);
   t1 = new TLatex(0.2,0.74,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();
   sprintf(string,"Bmax/Bsat=%.0f/%.0f\n",Bmax,BSaturation);
   t1 = new TLatex(0.2,0.70,string);
   t1->SetNDC();
   t1->SetTextSize(0.03);
   t1->Draw();

   TCanvas *c11 = new TCanvas("c11","Sketch of Shield and Coordinates",200,10,700,700);
   gPad->Range(-1.4*b,-1.4*b,1.4*b,1.4*b);
   c11->SetBorderMode(0);
   c11->SetFillColor(0);
   TEllipse *circle = new TEllipse(0,0,a);
   circle->SetLineColor(2);
   circle->Draw();
   TEllipse *circle = new TEllipse(0,0,b);
   circle->SetLineColor(2);
   circle->Draw();
   TArrow *arr = new TArrow(0,0,1.2*b,0,0.02,">");
   arr->Draw();
   TArrow *arr = new TArrow(0,0,0,1.2*b,0.02,">");
   arr->Draw();
   TArrow *arr = new TArrow(0,0,b/sqrt(2),b/sqrt(2),0.02,">");
   arr->SetLineColor(4);
   arr->Draw();
   t1 = new TLatex(1.3*b,0,"x");
   t1->SetTextSize(0.07);
   t1->Draw();
   t1 = new TLatex(0,1.3*b,"y");
   t1->SetTextSize(0.07);
   t1->Draw();
   t1 = new TLatex(1.1*b/sqrt(2),1.1*b/sqrt(2),"r");
   t1->SetTextSize(0.07);
   t1->Draw();
   t1 = new TLatex(.5*b,0.2*b,"#theta");
   t1->SetTextSize(0.07);
   t1->Draw();
   p1 = new TPaveLabel(-1.1*b,-1.1*b,-0.9*b,-0.9*b,"1");  
   p1->SetTextSize(0.5);
   p1->Draw();
   p1 = new TPaveLabel(-0.5*b,-0.5*b,-0.3*b,-0.3*b,"3");
   p1->SetTextSize(0.5);
   p1->Draw();
   p1 = new TPaveLabel(-0.6*b,-1.2*b,-0.4*b,-1.0*b,"2");
   p1->SetTextSize(0.5);
   p1->Draw();
   TArrow *arr = new TArrow(-0.5*b,-b,-0.38*b,-0.9*b,0.02,">");
   arr->SetLineColor(1);
   arr->SetLineWidth(2);
   arr->Draw();

   sprintf(string,"bshield_a%.2f_H0%.0f.eps",a,H0);
   c9->SaveAs(string);
   sprintf(string,"bshield_a%.2f_H0%.0f.gif",a,H0);
   c9->SaveAs(string);
   sprintf(string,"bshieldtot_a%.2f_H0%.0f.eps",a,H0);
   c10->SaveAs(string);
   sprintf(string,"bshieldtot_a%.2f_H0%.0f.gif",a,H0);
   c10->SaveAs(string);
   sprintf(string,"bshield_fg.gif");
   c11->SaveAs(string);
   sprintf(string,"bshield_fg.eps");
   c11->SaveAs(string);

   //
   // Before leaving this demo, we print the list of objects known to ROOT
   //
   //gObjectTable->Print();

}
Double_t Delta (Double_t *x, Double_t *par) {
// 
   // independent variable is the magnetic permeability

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t bsq=b*b;
   Double_t asq=a*a;

   Double_t D = bsq*(1+Km)*(1+Km)-asq*(1-Km)*(1-Km);
   if (D <= 0) printf("D=%f",D);
   return D;
}
Double_t B1 (Double_t *x, Double_t *par) {
// 
   // independent variable is the magnetic field strength

   Double_t Delta;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   Double_t d=Delta(x,par);
   Double_t ratio=-H0*bsq*(Km2-1)*(asq-bsq)/d;
   //printf("H0=%f, a=%f, b=%f, Km=%f, B1=%f, d=%f, ratio=%f\n",H0,a,b,Km,B1,d,ratio);
   return ratio;
}
Double_t B2M (Double_t *x, Double_t *par) {
// 
   // Negative to make quantitive positive.

   // independent variable is the magnetic field strength

   Double_t Delta;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   Double_t B2M=-2*H0*bsq*asq*(1-Km);
   Double_t d=Delta(x,par);
   Double_t ratio=B2M/d;
   //printf("H0=%f, a=%f, b=%f, Km=%f, B1=%f, d=%f, ratio=%f\n",H0,a,b,Km,B1,d,ratio);
   return ratio;
}
Double_t A2M (Double_t *x, Double_t *par) {
// 
   // Negative to make quantitive positive.

   // independent variable is the magnetic field strength

   Double_t Delta;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   Double_t A2M=2*H0*bsq*(1+Km);
   Double_t d=Delta(x,par);
   Double_t ratio=A2M/d;
   //printf("H0=%f, a=%f, b=%f, Km=%f, B1=%f, d=%f, ratio=%f\n",H0,a,b,Km,B1,d,ratio);
   return ratio;
}
Double_t A3M (Double_t *x, Double_t *par) {
// 
   // Negative to make quantitive positive.

   // independent variable is the magnetic field strength

   Double_t Delta;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   Double_t A3M=4*Km*H0*bsq;
   Double_t d=Delta(x,par);
   Double_t ratio=A3M/d;
   //printf("H0=%f, a=%f, b=%f, Km=%f, B1=%f, d=%f, ratio=%f\n",H0,a,b,Km,B1,d,ratio);
   return ratio;
}
Double_t Hx (Double_t *x, Double_t *par) {
// 
   // x component of Magnetic field strength H(x,y)
   // independent variables are the x and y positions, with the center of the tube at (0,0)
   // x = x[0]
   // y = x[1]

   Double_t Delta,B1,A2M,B2M,A3M;
   Double_t r,costhe,sinthe;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;
   Double_t H;

   // note that Bsaturation is also defined in main routine

   // double BSaturation=7600;  
   // divide by empirical factor of 2
   double BSaturation=3800;

   // printf ("x[0]=%f, x[1]=%f, par[3]=%f, H0=%f\n",x[0],x[1],par[3],H0);
   // printf("Hx - Km=%f\n",Km);

   // convert to polar coordinates

   r = sqrt(x[0]*x[0] + x[1]*x[1]);
   if (r > 0) {
      costhe = x[0]/r;
      sinthe = sqrt(1-costhe*costhe);
      if (x[1]<0) sinthe=-sinthe;}
   else {
      costhe = 1;
      sinthe = 0;}

   if (r >= b) {

      // outside shield 

      H = H0 + B1(x,par)*(costhe*costhe-sinthe*sinthe)/(r*r);
      }
   elseif (r >= a) {

      // in mu-metal

      H = A2M(x,par) -  B2M(x,par)*(costhe*costhe-sinthe*sinthe)/(r*r);
      // printf("H0=%f, r=%f, H=%f, H0/H=%f, Km=%f,\n",H0,r,H,H0/H,Km);
      }
   else {

      // inside magnetic shield, checking for saturation

      double Hmax = BSaturation*(bsq-asq)/(4*bsq);
      H = H0<Hmax?A3M(x,par):H0-Hmax;
      }

   // printf("H0=%f, r=%f, H=%f, H0/H=%f, Km=%f,\n",H0,r,H,H0/H,Km);
   return H;
}
Double_t Hy (Double_t *x, Double_t *par) {
// 
   // x component of Magnetic field strength H(x,y)
   // independent variables are the x and y positions, with the center of the tube at (0,0)
   // x = x[0]
   // y = x[1]

   Double_t Delta,B1,A2M,B2M,A3M;
   Double_t r,costhe,sinthe;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;
   Double_t H;

   // convert to polar coordinates

   r = sqrt(x[0]*x[0] + x[1]*x[1]);
   if (r > 0) {
      costhe = x[0]/r;
      sinthe = sqrt(1-costhe*costhe);
      if (x[1]<0) sinthe=-sinthe;
      //printf ("x[0]=%f,x[1]=%f, costhe=%f, sinthe=%f\n",x[0],x[1],costhe,sinthe);
      }
   else {
      costhe = 1;
      sinthe = 0;}

   if (r >= b) {

      // outside shield 

      H = 2*B1(x,par)*sinthe*costhe/(r*r);}
   elseif (r >= a) {

      // in mu-metal

      H = -2*B2M(x,par)*sinthe*costhe/(r*r);
      //printf ("r=%f, x=%f, y=%f, H=%f\n",r,x[0],x[1],H);
      }
   else {

      // inside magnetic shield

      H = 0;}

   // printf("H0=%f, r=%f, H=%f\n",H0,r,H);
   return H;
}
Double_t Vm (Double_t *x, Double_t *par) {
// 
   // Magnetic scalar potential Vm(x,y)
   // independent variables are the x and y positions, with the center of the tube at (0,0)
   // x = x[0]
   // y = x[1]

   Double_t Delta,B1,A2M,B2M,A3M;
   Double_t r,costhe;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   // convert to polar coordinates

   r = sqrt(x[0]*x[0]+x[1]*x[1]);
   if (r > 0) {
      costhe = x[0]/r;}
   else {
      costhe = 0;}

   // compute the scalar potential in the region outside the shield

   if (r > b) {
      Double_t Vm1 = -H0*r*costhe+B1(x,par)*costhe/r;}
   elseif (r > a) {
      Vm1 = 0;}
   else {
      Vm1 = 0;}

   //printf("H0=%f, a=%f, b=%f, Vm1=%f\n",H0,a,b,Vm1);
   return Vm1;
}
Double_t check1 (Double_t *x, Double_t *par) {
// 
   // independent variable is the magnetic field strength

   Double_t Delta,B1,A2M,B2M,A3M;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   Double_t check = -H0*bsq+B1(x,par)+A2M(x,par)*bsq+B2M(x,par);
   //printf("H0=%f, a=%f, b=%f, check1=%f\n",H0,a,b,check);
   return check;
}
Double_t check2 (Double_t *x, Double_t *par) {
// 
   // independent variable is the magnetic field strength

   Double_t Delta,B1,A2M,B2M,A3M;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   Double_t check = -A2M(x,par)*asq-B2M(x,par)+Km*A2M(x,par)*asq-Km*B2M(x,par);
   //printf("H0=%f, a=%f, b=%f, check2=%f\n",H0,a,b,check);
   return check;
}
Double_t check (Double_t *x, Double_t *par) {
// 
   // independent variable is the magnetic field strength

   Double_t Delta,B1,A2M,B2M,A3M;

   Double_t a=par[0];
   Double_t b=par[1];
   Double_t Km=par[2];
   Double_t H0=par[3];
   Double_t bsq=b*b;
   Double_t asq=a*a;
   Double_t Km2=Km*Km;

   Double_t check = -H0*bsq-B1(x,par)+Km*A2M(x,par)*bsq-Km*B2M(x,par);
   //printf("H0=%f, a=%f, b=%f, check3=%f\n",H0,a,b,check);
   return check;
}